package com.clockster.asgard_location_mock

import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import android.provider.Settings
import androidx.annotation.NonNull
import io.flutter.Log
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result


/** AsgardLocationMockPlugin */
class AsgardLocationMockPlugin: FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private var context:Context? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "asgard_location_mock")
    channel.setMethodCallHandler(this)
    context = flutterPluginBinding.applicationContext
  }



  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "getPlatformVersion") {
      result.success("Android ${android.os.Build.VERSION.RELEASE}")
    }else if(call.method == "isMockEnabled"){
      if(context == null){
        result.success(false)
      }else {
        result.success(isMockLocationOn(context!!))
      }
    } else {
      result.notImplemented()
    }
  }

  private fun isMockLocationOn(context: Context): Boolean {
    return if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
      "0" == Settings.Secure.getString(context.contentResolver, Settings.Secure.ALLOW_MOCK_LOCATION)
    } else {
      val pm: PackageManager = context.packageManager
      val packages = pm.getInstalledApplications(PackageManager.GET_META_DATA)
      for (applicationInfo in packages) {
        try {
          val packageInfo = pm.getPackageInfo(applicationInfo.packageName,
                  PackageManager.GET_PERMISSIONS)

          // Get Permissions
          val requestedPermissions = packageInfo.requestedPermissions
          if (requestedPermissions != null) {
            for (i in requestedPermissions.indices) {
              if ((requestedPermissions[i]
                              == "android.permission.ACCESS_MOCK_LOCATION") && applicationInfo.packageName != context.packageName) {
                return true
              }
            }
          }
        } catch (e: PackageManager.NameNotFoundException) {
          Log.e("Mock location check error ", "Mock location check error ")
        }
      }
      false
    }
  }



  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
    context = null
  }
}
